'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');

/* global Reflect, Promise */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var registryContext = React.createContext({});
var RegistriesConsumer = registryContext.Consumer;
var RegistryProvider = registryContext.Provider;
function withRegistry() {
    // Use arguments instead of rest-arguments to get faster and more compact code.
    var registries = [].slice.call(arguments);
    return function WithRegistry(Component) {
        var RegistryResolver = function (props) {
            var providedRegistriesRef = React.useRef(null);
            return (React.createElement(RegistriesConsumer, null, function (contextRegistries) {
                if (providedRegistriesRef.current === null) {
                    var providedRegistries = __assign({}, contextRegistries);
                    for (var i = 0; i < registries.length; i++) {
                        var registry = registries[i];
                        var overrides = providedRegistries[registry.id];
                        // eslint-disable-next-line no-nested-ternary
                        providedRegistries[registry.id] = registry.overridable
                            ? overrides
                                ? registry.merge(overrides)
                                : registry
                            : registry && overrides
                                ? overrides.merge(registry)
                                : registry;
                    }
                    providedRegistriesRef.current = providedRegistries;
                }
                return (React.createElement(RegistryProvider, { value: providedRegistriesRef.current }, React.createElement(Component, props)));
            }));
        };
        {
            var resolverValue = registries.map(function (registry) { return registry.id; }).join(', ');
            // TODO: Use setDisplayName util.
            RegistryResolver.displayName = "RegistryResolver(".concat(resolverValue, ")");
        }
        return RegistryResolver;
    };
}
var RegistryConsumer = function (props) { return (React.createElement(RegistriesConsumer, null, function (registries) {
    {
        if (!registries[props.id]) {
            throw new Error("Registry with id '".concat(props.id, "' not found."));
        }
    }
    return props.children(registries[props.id].snapshot());
})); };
/**
 * @deprecated consider using 'RegistryConsumer' instead
 */
var ComponentRegistryConsumer = RegistryConsumer;
var useRegistries = function () {
    return React.useContext(registryContext);
};
function useRegistry(id) {
    var registries = useRegistries();
    return registries[id].snapshot();
}
/**
 * @deprecated consider using 'useRegistry' instead
 */
var useComponentRegistry = useRegistry;
var registryOverloadMark = 'RegistryOverloadMark';
function withOverload(overload) {
    return {
        $symbol: registryOverloadMark,
        overload: overload,
    };
}
function isOverload(entity) {
    return entity.$symbol === registryOverloadMark;
}
var Registry = /** @class */ (function () {
    function Registry(_a) {
        var id = _a.id, _b = _a.overridable, overridable = _b === void 0 ? true : _b;
        this.entities = {};
        this.id = id;
        this.overridable = overridable;
    }
    /**
     * Set registry entry by id.
     *
     * @param id entry id
     * @param entity valid registry entity
     */
    Registry.prototype.set = function (id, entity) {
        this.entities[id] = entity;
        return this;
    };
    /**
     * Set extender for registry entry by id.
     *
     * @param id entry id
     * @param overload valid registry entity extender
     */
    Registry.prototype.extends = function (id, overload) {
        this.entities[id] = withOverload(overload);
        return this;
    };
    /**
     * Set react entities in registry via object literal.
     *
     * @param entitiesSet set of valid registry entities
     */
    Registry.prototype.fill = function (entitiesSet) {
        for (var key in entitiesSet) {
            this.entities[key] = entitiesSet[key];
        }
        return this;
    };
    /**
     * Get entry from registry by id.
     *
     * @param id entry id
     */
    Registry.prototype.get = function (id) {
        {
            if (!this.entities[id]) {
                throw new Error("Entry with id '".concat(id, "' not found."));
            }
        }
        return this.entities[id];
    };
    /**
     * Returns raw entities from registry.
     */
    Registry.prototype.snapshot = function () {
        return this.entities;
    };
    /**
     * Override entities by external registry.
     * @internal
     *
     * @param otherRegistry external registry
     */
    Registry.prototype.merge = function (otherRegistry) {
        var clone = new Registry({ id: this.id, overridable: this.overridable });
        clone.fill(this.entities);
        if (!otherRegistry)
            return clone;
        var otherRegistryEntities = otherRegistry.snapshot();
        for (var entityName in otherRegistryEntities) {
            if (!otherRegistryEntities.hasOwnProperty(entityName))
                continue;
            clone.entities[entityName] = this.mergeEntities(clone.entities[entityName], otherRegistryEntities[entityName]);
        }
        return clone;
    };
    /**
     * Returns extended or replaced entity
     *
     * @param base base implementation
     * @param overrides overridden implementation
     */
    Registry.prototype.mergeEntities = function (base, overrides) {
        if (isOverload(overrides)) {
            if (!base)
                return overrides;
            if (isOverload(base)) {
                // If both entities are hocs, then create compose-hoc
                return withOverload(function (Base) { return overrides.overload(base.overload(Base)); });
            }
            return overrides.overload(base);
        }
        return overrides;
    };
    return Registry;
}());

exports.ComponentRegistryConsumer = ComponentRegistryConsumer;
exports.Registry = Registry;
exports.RegistryConsumer = RegistryConsumer;
exports.registryContext = registryContext;
exports.useComponentRegistry = useComponentRegistry;
exports.useRegistries = useRegistries;
exports.useRegistry = useRegistry;
exports.withRegistry = withRegistry;
